# ESA (C) 2000-2022
# 
# This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
#
#    SAS is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    SAS is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

# xmmextractorGUIpy.py

from .version import VERSION, SAS_RELEASE, SAS_AKA
import os
from os import environ
import shutil, sys
import stat
from xml.dom import minidom
import traceback
from urllib.request import Request, urlopen
import requests
import tarfile
#from os import environ
#from os.path import expanduser, basename
from pathlib import Path
from PyQt5 import QtWidgets
from PyQt5.QtGui import QFont, QFontMetrics
from PyQt5.QtWidgets import QHBoxLayout, QTabWidget,QMainWindow,QPushButton, QVBoxLayout,\
    QLineEdit, QFormLayout, QMessageBox, QButtonGroup, QAbstractButton
from PyQt5.Qt import QApplication, QWidget, QPixmap, QLabel, QScrollArea,QFrame,QGroupBox,\
    QGridLayout, QSizePolicy, QLayout, QFileDialog, QObject, QDialog, QComboBox
from PyQt5.QtCore import QRect, QTimer,pyqtSlot, Qt, QFile, QTextStream


# XMMextractorpy utils
from pysas.XMMextractorpy import XMMextractorpy
from pysas.pyOAL import pyOAL
from pysas.logger import TaskLogger as TL
from pysas.xmmextractorGUIpy.controlPanel import controlPanel
from pysas.xmmextractorGUIpy.showImageClass import PlotCanvas
#TO DO: TO BE INCLUDED (AG)
#from pysas.pyutils.xmmextractor_tools  import xsareqest

__version__ = f'xmmextractorGUIpy (xmmextractorGUIpy-{VERSION}) [{SAS_RELEASE}-{SAS_AKA}]' 

# The Python code for xmmextractorGUIpy should go inside run


# The Python code for xmmextractorGUI should go inside run
class App(QMainWindow):
 
    def __init__(self,fileNameList):
        super().__init__()
        self.title = 'xmmextractor'
        self.left = 0
        self.top = 0
        self.width = 1600
        self.height = 1000
        self.setWindowTitle(self.title)
        self.setGeometry(self.left, self.top, self.width, self.height)
               
        DownloadODFFlag = False
        templateXMLFileName = ""

        if fileNameList[0] == 'none':
           #Find the file in the SAS_DIR data directory or SAS_PATH...
            XMLFileSASDIR = Path(str(os.environ['SAS_DIR'])+'/lib/data/xmlTemplateData/xmmextractorDEFAULTParam.xml') 
            saspath = str(os.environ['SAS_PATH']).split(":")
            XMLFileSASPATH = Path(saspath[0]+'/lib/data/xmlTemplateData/xmmextractorDEFAULTParam.xml') 
            if (XMLFileSASDIR.is_file()):
                self.xmldocFile = str(os.environ['SAS_DIR'])+'/lib/data/xmlTemplateData/xmmextractorDEFAULTParam.xml'               
            elif XMLFileSASPATH.is_file():
                self.xmldocFile = saspath[0]+'/lib/data/xmlTemplateData/xmmextractorDEFAULTParam.xml'
            else:
                print("Default XML template file nor found")

            templateXMLFileName="/tmp/"+str(os.getpid())+"_xmmextractorDEFAULTParam.xml"
            shutil.copyfile( self.xmldocFile, templateXMLFileName)            
            self.xmldocFile = templateXMLFileName
            os.chmod(self.xmldocFile, stat.S_IRUSR | stat.S_IWUSR | stat.S_IRGRP | stat.S_IWGRP |stat.S_IROTH | stat.S_IWOTH )
            DownloadODFFlag = True
        else:
            #Check if the file exists
            if  Path(fileNameList[0]).is_file():
                self.xmldocFile = fileNameList[0]
            else:
                print('error XML File '+fileNameList[0]+" does not exists")
                exit(1)
                    
        self.table_widget = mainPanel(self, self.xmldocFile,DownloadODFFlag,templateXMLFileName)
        self.setCentralWidget(self.table_widget)
        
        self.show()




class mainPanel(QWidget):
    
    def __init__(self,parent,xmldocFile,downloadFlag,templateXMLFileName):
        super(QWidget, self).__init__(parent)
        
        #logger.log('info', ('Creating main panel'))
                   
        self.XMMTimer = None
                   
        self.xmldocFile = xmldocFile
        self.templateXMLFileName = templateXMLFileName
        print(self.xmldocFile)

        print(self.xmldocFile)

        if  'DEFAULTParam' in  self.xmldocFile:
            self.xmldoc = minidom.parse(self.xmldocFile)            
        else:
            #TO DO!!!!
            self.obs = pyOAL.Observation(self.xmldocFile)
        self.DownloadODFFlag = downloadFlag
        #observation info
        self.obsInfoDict = dict()
        #procInfo
        self.procEPNDict = dict()
        self.procEMOS1Dict = dict()
        self.procEMOS2Dict = dict()
        self.procRGS1Dict = dict()
        self.procRGS2Dict = dict()
        self.procOMDict = dict()
        
        self.EPNExpoInfoDict = dict()
        self.EMOS1ExpoInfoDict = dict()
        self.EMOS2ExpoInfoDict = dict()
        self.RGS1ExpoInfoDict = dict()
        self.RGS2ExpoInfoDict = dict()
        self.OMExpoInfoDict = dict()
        
        self.tasksEPNDict = dict()
        self.tasksEMOS1Dict = dict()
        self.tasksEMOS2Dict = dict()
        self.tasksRGS1Dict = dict()
        self.tasksRGS2Dict = dict()
        self.tasksOMDict = dict()
        
        self.expoTabProductDict = dict()
        self.paramsDict = dict()
        self.sn = 0.0
        self.bkgCR = 0.0
        self.SNValDict = dict()
        self.SNBKGValDict = dict()
        self.prod=""
        self.thresholdDict = dict()
        self.pid = None
        self.loginWidget = None
        #...Layout directory. Needed to refresh graphics


        #Ds9 instance
        self.p = None

        #Check SAS variables....
        self.checkSASEnviromentalVariables()

        #Parse XML file        
        self.parseXMLFile("init")
        #Screen

        self.layout = QVBoxLayout(self) 

        #Left block
        leftFrame = QFrame(self)
        leftFrame.setMinimumWidth(500)
        leftLayout = QVBoxLayout()
        leftFrame.setLayout(leftLayout)
        #Download Frame
        leftLayout.addWidget(self.getDownloadFrame())
        #Observation info
        leftLayout.addWidget(self.getObservationTab())
        #Output directory widgets need to be created
        #before ccf.cif and SUM.SAS exists
        outputGroupBox = QGroupBox(leftFrame)
        outputGroupBox.setTitle("Output Directory")
        outputLayout = QFormLayout()
        outputLabel = QLabel("Output Directory")
        self.outputField = QLineEdit()
        self.outputField.setText(os.getcwd())
        outputLayout.addRow(outputLabel, self.outputField)
        fileSearchLabel = QLabel("Search")
        fileDialogButton = QPushButton("Change Dir")
        fileDialogButton.clicked.connect(self.getDirectory)
        outputLayout.addRow(fileSearchLabel, fileDialogButton)
        outputGroupBox.setLayout(outputLayout)
        
        
        #Configuration Group Box
        confGroupBox = QGroupBox(leftFrame)
        confGroupBox.setTitle("Configuration")
        confLayout = QFormLayout()

        ccfLabel = QLabel("CCF File")
        self.ccfField = QLineEdit()            
        self.ccfField.setText(self.getFileName('.cif',str(self.outputField.text())))
        confLayout.addRow(ccfLabel, self.ccfField)

        odfLabel = QLabel("SUM.SAS File")
        self.odfField = QLineEdit()        
        self.odfField.setText(self.getFileName('SUM.SAS',str(self.outputField.text())))
        confLayout.addRow(odfLabel, self.odfField)
           
        confGroupBox.setLayout(confLayout)     
        leftLayout.addWidget(confGroupBox)
        #
        # Ouptut directory Group Box                       
        leftLayout.addWidget(outputGroupBox)
        #End of output directory Group Box
        #End of Left box
        
        #Initialize tab screen
        self.tabs = QTabWidget()  


        #Main tab
        self.MainTab = QWidget()
        self.MainTab.layout = QHBoxLayout()
        self.MainTab.layout.addWidget(leftFrame,1)
        

        self.epicImageFrame =  QFrame()
        self.epicImageLayout = QVBoxLayout()
        self.epicImageFrame.setLayout(self.epicImageLayout)



        if self.obsInfoDict['obsid'] != "":
            if (self.getEPICImage() == 'NOT FOUND'):            
                err = self.xsaRequest('IMA',self.obsInfoDict['obsid'],"","")
                self.readSUMASCFile()

        #Check if there is a ODF directoy in the current dir. If yes, read the SUM.ASC file and check if there is an
        #EPIC image
        odfPath = str(self.outputField.text())+'/ODF/'
        if os.path.isdir(odfPath) and self.obsInfoDict['obsid'] == "":
            self.readSUMASCFile()
            self.AIOButton.setEnabled(False)
            self.DownloadODFFlag = False
            # Set SAS_ODF...
            os.environ["SAS_ODF"]=odfPath


        if ( self.getEPICImage() != 'NOT FOUND'):
            if len((self.odf_ra_data.text())) == 0 :
                self.readSUMASCFile()
            ra = float(self.obsInfoDict['ra'])
            dec = float(self.obsInfoDict['dec'])
            #ra=float(self.odf_ra_data.text())
            #dec=float(self.odf_dec_data.text())
            self.epicImageLayout.addWidget(PlotCanvas(self, width=20, height=20,fileName=self.getEPICImage(),type='IM',ra=ra,dec=dec),4)             
        self.MainTab.layout.addWidget(self.epicImageFrame,1)
        self.tabs.addTab(self.MainTab, "Main")
        self.MainTab.setLayout(self.MainTab.layout)
        #self.ObsTab.layout.addWidget(PlotCanvas(self, width=5, height=5),2)
        #End of Main Tab

        #Observation Tab
        self.ObsTab = QWidget()                        
        self.ObsTab.layout = QHBoxLayout()        

        #self.ObsTab.layout.addWidget(self.getObservationTab(),1)
        #self.ObsTab.layout.addWidget(leftFrame,1)    

        #Right Frame
        rightFrame = QFrame()
        self.rightLayout = QVBoxLayout()
        rightFrame.setLayout(self.rightLayout)

        self.rightLayout.addWidget(self.getProductsTitle(),1)
        self.rightLayout.addStretch(0)
        self.controlButtonsFrame =  self.createControlPanel()
        self.rightLayout.addWidget(self.controlButtonsFrame,2)
               
        self.rightLayout.addWidget(self.addInstrumentExpoInfo(),1)
    
        self.ObsTab.layout.addWidget(rightFrame,3)
        
            


        self.tabs.addTab(self.ObsTab, "Settings")
        self.ObsTab.setLayout(self.ObsTab.layout)

        #if self.DownloadODFFlag == False: 
        #    self.createInstrumentTabs()
  
        #Add tabs to Widget
        self.layout.addWidget(self.tabs)    
        self.layout.addWidget(self.getControlButtons(self))
        #Slot to check the focus on tabs
        self.tabs.currentChanged.connect(self.tabSelected)
    

        if self.DownloadODFFlag:
            self.runButton.setDisabled(True)




            
    def checkSASEnviromentalVariables(self):
        if "SAS_CCFPATH" not in os.environ: 
            print("SAS_CCFPATH undefined. Please set the enviromental variable SAS_CCFPATH")
            exit(1)




    def parseXMLFile(self,level):
        #self.logger.info("Parsing XML file")
        if level == "process":
            self.xmldocFile = self.getFileName("xmmextractorParam.xml", str(self.outputField.text()))
            self.xmldoc = minidom.parse(self.xmldocFile)
            itemlist = self.xmldoc.getElementsByTagName('OBSERVATION')
        else:
            itemlist = self.xmldoc.getElementsByTagName('OBSERVATION')
        #obsItemList = itemlist[0].getElementsByTagName('PARAM')
        obsItemList = itemlist[0].getElementsByTagName('PARAM')
        for element in obsItemList:
            self.obsInfoDict[element.getAttribute('id')] = element.getAttribute('default') 
            if element.getAttribute('id') == "EPN":
                self.procEPNDict['Instrument'] = element.getAttribute('id')
                self.procEPNDict['Processing'] = element.getAttribute('default')                    
            elif element.getAttribute('id') == "EMOS1":
                self.procEMOS1Dict['Instrument'] = element.getAttribute('id')
                self.procEMOS1Dict['Processing'] = element.getAttribute('default')                
            elif element.getAttribute('id') == "EMOS2":
                self.procEMOS2Dict['Instrument'] = element.getAttribute('id')
                self.procEMOS2Dict['Processing'] = element.getAttribute('default')
            elif element.getAttribute('id') == "RGS1":
                self.procRGS1Dict['Instrument'] = element.getAttribute('id')
                self.procRGS1Dict['Processing'] = element.getAttribute('default')
            elif element.getAttribute('id') == "RGS2":
                self.procRGS2Dict['Instrument'] = element.getAttribute('id')
                self.procRGS2Dict['Processing'] = element.getAttribute('default')
            elif  element.getAttribute('id') == "OM":
                self.procOMDict['Instrument'] = element.getAttribute('id')
                self.procOMDict['Processing'] = element.getAttribute('default')       
            else:
                print("Keyword not found "+element.getAttribute('id'));
        
        instItemList = self.xmldoc.getElementsByTagName('INSTRUMENT')
        i = 0
        j = 0
        for element in instItemList:
            instrument = element.getAttribute('value')
            expItemList = instItemList[i].getElementsByTagName('EXPOSURE')
            i = i + 1
            for expElement in  expItemList:
                expInfo = dict()
                expBrowserInfo = dict()       
                expId = expElement.getAttribute('expid')
                expBrowserInfo['duration'] = expElement.getAttribute('duration') 
                expBrowserInfo['mode'] = expElement.getAttribute('mode')
                
                expInfo['Process'] = expElement.getAttribute('process')  
                   
                productItemList = expElement.getElementsByTagName('PRODUCT')
                productInfo = dict()
                j = j + 1        
                for taskElement in productItemList:
                    product = taskElement.getAttribute('value')
                    taskProc = taskElement.getAttribute('process')   
                    expInfo[product] = taskProc
                    taskList = taskElement.getElementsByTagName('TASK')   
                    productDict = dict()  
                    #productPurpose = dict()  
                    taskCounter = 1  
                    for task in taskList:
                        #print("TASK_"+str(taskCounter)+" "+task.getAttribute('name'))
                        productDict["task_"+str(taskCounter)] = task.getAttribute('name')+"%"+task.getAttribute('purpose')
                        
                        paramList = task.getElementsByTagName('PARAM')
                        paramInfo = dict()
                        for param in paramList:
                            #print(param.getAttribute('id')+'='+param.getAttribute('default'))
                            paramInfo[param.getAttribute('id')] = param.getAttribute('default')
                        productDict["params_"+str(taskCounter)] = paramInfo 
                        taskCounter = taskCounter+1
                        #print(paramInfo) 
                    #print(productDict)
                        #productPurpose[task.getAttribute('purpose')] =  productDict   
                    productInfo[product] = productDict

                    #productInfo[product] =   productPurpose  
                #print(productInfo)                                

                if instrument == "EPN":
                    #print(expInfo)
                    #print(productInfo)
                    #print(expId)
                    self.procEPNDict[expId] =  expInfo
                    self.tasksEPNDict[expId] = productInfo
                    self.EPNExpoInfoDict[expId] = expBrowserInfo
                if instrument == "EMOS1":
                    self.procEMOS1Dict[expId] =  expInfo
                    self.tasksEMOS1Dict[expId] = productInfo
                    self.EMOS1ExpoInfoDict[expId] = expBrowserInfo
                if instrument == "EMOS2":
                    self.procEMOS2Dict[expId] =  expInfo
                    self.tasksEMOS2Dict[expId] = productInfo
                    self.EMOS2ExpoInfoDict[expId] = expBrowserInfo                                  
                if instrument == "RGS1":
                    self.procRGS1Dict[expId] =  expInfo
                    self.tasksRGS1Dict[expId] = productInfo
                    self.RGS1ExpoInfoDict[expId] = expBrowserInfo                                        
                if instrument == "RGS2":
                    self.procRGS2Dict[expId] =  expInfo
                    self.tasksRGS2Dict[expId] = productInfo
                    self.RGS2ExpoInfoDict[expId] = expBrowserInfo                                        
                if instrument == "OM":
                    self.procOMDict[expId] =  expInfo
                    self.tasksOMDict[expId] = productInfo
                    self.OMExpoInfoDict[expId] = expBrowserInfo                      
        



    def getDownloadFrame(self):
    
        horizontalGroupBox = QGroupBox("Download ODF")
        odf_download_layout = QFormLayout() 
        odf_download_info = QFrame()
        tagDownloadLayout = QHBoxLayout()
        self.obsidLabel = QLabel('obsid')
        self.AIOButton = QPushButton()
        self.AIOButton.setText("Download ODF")      
        if self.DownloadODFFlag == False:
            self.AIOButton.setDisabled(True)
                    #tagLayout.addRow(label,AIOButton)
        tagDownloadLayout.addWidget(self.obsidLabel)
        self.odf_obsid_data = QLineEdit();
        self.odf_obsid_data.setToolTip("Current Observation ID to be processed")                                    
        self.AIOButton.clicked.connect(self.downloadODF)
        tagDownloadLayout.addWidget(self.odf_obsid_data)
        tagDownloadLayout.addWidget(self.AIOButton)
        odf_download_info.setLayout(tagDownloadLayout)  
        odf_download_layout.addRow(odf_download_info)       
        horizontalGroupBox.setLayout(odf_download_layout)        
        
        #Check if there is internet connection.
        if self.online(4) == False:
            self.AIOButton.setEnabled(False)
        
        
        return horizontalGroupBox


    def online(self,timeout):
        req = Request("http://www.google.com")
        try:
            response = urlopen(req)
        except HTTPError as e:
            #self.logger.info('No internet connection')
            #self.logger.error('Error code: ', e.code)
            return False
        except URLError as e:
            #self.logger.info('No internet connection.')
            #self.logger.error('Reason: ', e.reason)
            return False
        else:
            #self.logger.info('Internet connection fine')
            return True

    def downloadODF(self):
        #self.logger.info("Downloading ODF file")
        obsid = self.odf_obsid_data.text()        
        
        if not obsid:        
            msg = QMessageBox()
            msg.setWindowTitle("Observation ID")
            msg.setIcon(QMessageBox.Warning)
            msg.setText("ObsID empty")
            msg.setStandardButtons(QMessageBox.Ok)
            msg.buttonClicked.connect(self.msgbtn)
            msg.exec_()
        else:
            self.loginWidget = QDialog(self)
            self.loginWidget.setWindowTitle("ODF Download")
            xsaDetails = QFrame()
            uNameLayout = QFormLayout()
            xsaDetails.setLayout(uNameLayout)
            uName = QLabel("XSA login")
            self.userName = QLineEdit(self.loginWidget)
            uNameLayout.addRow(uName, self.userName)
            passw= QLabel("password")
            self.passWord = QLineEdit(self.loginWidget)
            uNameLayout.addRow(passw, self.passWord)
            self.passWord.setEchoMode(QLineEdit.Password)
            self.buttonLogin = QPushButton('Login', self)
            self.buttonLogin.clicked.connect(self.handleLogin)       
            self.buttonCancel = QPushButton('Cancel', self)
            self.buttonCancel.clicked.connect(self.handleCancel)       
            loginLayout = QVBoxLayout(self.loginWidget)
            #loginLayout.addWidget(self.userName)
            loginLayout.addWidget(xsaDetails)
            #loginLayout.addWidget(self.passWord)
            loginLayout.addWidget(self.buttonLogin)  
            loginLayout.addWidget(self.buttonCancel)  
            val = self.loginWidget.exec_()    
            err = 1
            if val == 1:
                err = self.xsaRequest('ODF',obsid,self.userName.text(),self.passWord.text())
                if err == 1 and val == 1:       
                    self.unpackODF()            
                    self.readSUMASCFile()
                    err = self.xsaRequest('IMA',obsid,self.userName.text(),self.passWord.text())

                    if  self.obsInfoDict['ra']:
                        ra=float(self.obsInfoDict['ra'])
                        if  self.obsInfoDict['dec']:
                            dec=float( self.obsInfoDict['dec'])
                            self.epicImageLayout.addWidget(PlotCanvas(self, width=20, height=20,fileName=self.getEPICImage(),type='IM',ra=ra,dec=dec),4) 
                            


    def getObservationTab(self):
        #self.logger.info("Creating Observation Tab....")        
                        
        
        horizontalGroupBox = QGroupBox("ODF")

        odf_layout = QFormLayout() 
        for key,val in self.obsInfoDict.items():
            if key != "analysisoption"  and \
            key != "OM_sourcematch" and key != "EPN" and key != "EMOS1" and\
            key != "EMOS2" and key != "RGS1" and key != "RGS2" and key != "OM":                 
                #odf_info = QLabel(key); 
                #odf_info = QObject()
                if key == "obsid":
                    odf_info = QFrame()
                    if self.DownloadODFFlag == False:
                        self.AIOButton.setDisabled(True)

                    self.odf_obsid_data.setText(val);
                else:
                    odf_info = QLabel(key)
                      
                
                if key == "epicsrc":
                    self.odf_epicsr_data = QComboBox();
                    #self.odf_epicsr_data.addItem(val);
                    self.odf_epicsr_data.addItem('Proposal Central Target');
                    self.odf_epicsr_data.addItem('Central Target');
                    self.odf_epicsr_data.addItem('User\'s Defined');
                    if val == "no":
                        self.odf_epicsr_data.setCurrentIndex(1)   
                    self.odf_epicsr_data.currentIndexChanged.connect(self.EpicSrcSelectionchange)
                    self.odf_epicsr_data.setToolTip("Use proposal target coordinates")                    
                    self.odf_epicsr_data.setMinimumWidth(200)     
                    odf_layout.addRow(odf_info,self.odf_epicsr_data)
                elif key == "ra":
                    self.odf_ra_data = QLineEdit(val);
                    self.odf_ra_data.setToolTip("Source coordinates (RA,DEC) to extract products")
                    self.odf_ra_data.setMinimumWidth(200)
                    odf_layout.addRow(odf_info,self.odf_ra_data)
                    self.odf_ra_data.setEnabled(False)
                elif key == "dec":
                    self.odf_dec_data = QLineEdit(val);
                    self.odf_dec_data.setToolTip("Source coordinates (RA,DEC) to extract products")
                    self.odf_dec_data.setMinimumWidth(200)
                    odf_layout.addRow(odf_info,self.odf_dec_data)
                    self.odf_dec_data.setEnabled(False)
                elif key == "sourcename":
                    self.odf_sourcename_data = QLineEdit(val);
                    self.odf_sourcename_data.setToolTip("Label to tag all xmmextractor source products")
                    self.odf_sourcename_data.setMinimumWidth(200)
                    odf_layout.addRow(odf_info,self.odf_sourcename_data)                                                                                                                        
            else:
                print("Keyword not found "+val);                
                
        horizontalGroupBox.setLayout(odf_layout)        
        
        return horizontalGroupBox



    def EpicSrcSelectionchange(self):

        #for count in range(self.odf_epicsr_data.count()):
        #    print (self.odf_epicsr_data.itemText(count))
        #self.logger.info("Current index "+str(count)+" selection changed "+self.odf_epicsr_data.currentText() )
        if self.odf_epicsr_data.currentText() == 'Proposal Central Target':
            self.odf_epicsr_data.setToolTip("Use proposal target coordinates")     
            self.cp.activateSrcDetection(True)
            self.odf_ra_data.setText(str(self.obsInfoDict['ra']))
            self.odf_dec_data.setText(str(self.obsInfoDict['dec']))
            self.odf_ra_data.setEnabled(False)
            self.odf_dec_data.setEnabled(False)
        elif self.odf_epicsr_data.currentText() == 'User\'s Defined':
            self.odf_epicsr_data.setToolTip("Use User's coordinates")     
            self.cp.activateSrcDetection(True)  
            self.odf_ra_data.setText("")
            self.odf_dec_data.setText("")            
            self.odf_ra_data.setEnabled(True)
            self.odf_dec_data.setEnabled(True)
        else:
            self.odf_epicsr_data.setToolTip("Run source detection and automatically identify the central source ")           
            self.cp.activateSrcDetection(False)
            self.cp.changeButtonState(self.procEPNDict,False)
            self.cp.changeButtonState(self.procEMOS1Dict,False)
            self.cp.changeButtonState(self.procEMOS2Dict,False)
            self.odf_ra_data.setText("")
            self.odf_dec_data.setText("")            
            self.odf_ra_data.setEnabled(False)
            self.odf_dec_data.setEnabled(False)



    def getDirectory(self):
        searchDialog = QFileDialog()
        currDir = self.outputField.text()
        newDir = searchDialog.getExistingDirectory(self, 'Open a Folder', expanduser("~"),QFileDialog.ShowDirsOnly)
        if len(newDir) == 0:
            self.outputField.setText(currDir)            
        else:
            self.outputField.setText(newDir)            
            #self.logger.info("New directory selected "+newDir)



    def getFileName(self,fileExtension,dirPath):
        #self.logger.info("Search file name with pattern... "+ fileExtension)
        files = os.listdir(dirPath)
        fileName = [i for i in files if i.endswith(fileExtension)]
        if len(fileName) == 0:
            file = "NOT FOUND"
        else:
            file = dirPath+'/'+str(fileName[0])
        
        #self.logger.info("Retrieving file with name "+ file)
        
        return file




    def getEPICImage(self):
        #First Check if the file exists...
        obsid = self.odf_obsid_data.text() 
        epicImageFileName = str(self.outputField.text())+"/P"+str(obsid)+"EPX000OIMAGE8000.FTZ"

        if (Path(epicImageFileName).is_file()):
            return epicImageFileName
        else:
            return 'NOT FOUND'


    #TO DO: TO BE DELETED AND REPLACE BY xmmextrator_tools (AG)
    def xsaRequest(self,product,obsid,userName, psw):
        err = 1
        try:
            command = None
            outFile = None
            #self.logger.info("Downloading ODF....")

            if (userName != ""):                    
                if product == 'ODF':
                    command = "http://nxsa.esac.esa.int/nxsa-sl/servlet/data-action-aio?obsno="+str(obsid)+"&level=ODF&AIOUSER="+userName+"&AIOPWD="+psw                
                else:
                    command = "http://nxsa.esac.esa.int/nxsa-sl/servlet/data-action-aio?obsno="+str(obsid)+"&name=OIMAGE&level=PPS&extension=FTZ&AIOUSER="+userName+"&AIOPWD="+psw
            else:
                if product == 'ODF':
                    command = "http://nxsa.esac.esa.int/nxsa-sl/servlet/data-action-aio?obsno="+str(obsid)+"&level=ODF"               
                else:
                    command = "http://nxsa.esac.esa.int/nxsa-sl/servlet/data-action-aio?obsno="+str(obsid)+"&name=OIMAGE&level=PPS&extension=FTZ"
                
            if product == 'ODF':
                tarFileName = str(self.outputField.text())+"/"+str(obsid)+'.tar.gz'
                outFile = tarFileName
            else:
                outFile = str(self.outputField.text())+"/P"+str(obsid)+"EPX000OIMAGE8000.FTZ"

            r = requests.get(command,stream=True)

            if  r.status_code == 401 or r.status_code == 404 or r.status_code == 500:
                self.loginError = QDialog()
                self.loginError.setWindowTitle("ODF Download Error")                    
                loginMessages = QLineEdit(self.loginError)
                text = "Proprietary Data"
                
                if product == 'ODF':
                    if  r.status_code == 404:
                        text="Check ObsID number"
                    if  r.status_code == 401:
                        text="Check credentials"

                if product == 'IMA':
                    if  r.status_code == 404:
                        text="EPIC Image not found"

                if r.status_code == 500:
                    text="internal Error. Please try later"
                
                   
                loginMessages.setText(text)
                #loginMessages.setMinimumWidth(100)
                buttonLoginErrorOK = QPushButton('OK', self)
                buttonLoginErrorOK.clicked.connect(self.handleError)       
                loginLayout = QVBoxLayout(self.loginError)
                loginLayout.addWidget(loginMessages)
                loginLayout.addWidget(buttonLoginErrorOK)  
                err = self.loginError.exec_()    

                    
            if r.status_code == 200:
                with open(outFile, 'wb') as f:
                    shutil.copyfileobj(r.raw, f)
                    #self.logger.info("STATUS "+str(r.status_code))   

            # Check if the file is empty or does not exists....
            try:
                if os.path.getsize(outFile) > 0:
                    print("Donwloading EPIC Iamge...")                
                else:
                    msg = QMessageBox()
                    msg.setWindowTitle("XSA Response")
                    msg.setIcon(QMessageBox.Warning)
                    msg.setText("XSA File Empty. Please, check credentials")
                    msg.setStandardButtons(QMessageBox.Ok)
                    msg.buttonClicked.connect(self.msgbtn)
                    msg.exec_()
                    self.status = -1
                    err = self.status
            except OSError as e:
                print ("FILE DOES NOT EXIST")
                self.status = -1
                err = self.status
     
        except:
            self.error = traceback.format_exc()
            self.status = -1
            print("Error downloading ODF file "+ str(self.error))
            err = self.status
            
        return err




    def readSUMASCFile(self):
        odfDir = None
        if environ.get('SAS_ODF') is not None and Path(environ.get('SAS_ODF')).is_file():
            odfDir =self.getODFPathFromSUMSAS( os.environ['SAS_ODF'])       
        else:
            odfDir = str(self.outputField.text())+'/ODF/'

        sumASCFile = str(self.getFileName('SUM.ASC',odfDir.rstrip()))

        f = open(sumASCFile, 'r') # 'r' = read
        for line in f:
            if ('Observation/Slew Identifier' in line):
                val = line.split('/')
                val[0] = val[0].rstrip()
                self.odf_obsid_data.setText(val[0]) 
            if ('Target Name' in line):
                val = line.split('/')
                val[0] = val[0].rstrip()
                self.odf_sourcename_data.setText(val[0]) 
            if ('Target Right Ascension' in line):
                val = line.split('/')
                ra = round((float(val[0])*180.)/12.,5)
                #self.odf_ra_data.setText(str(ra))
                self.obsInfoDict['ra']=ra
                self.odf_epicsr_data.setItemText(0,"Proposal Central Target")
            if ('Target Declination' in line):
                val = line.split('/')
                dec = float(val[0])
                dec = round(dec,5)
                #self.odf_dec_data.setText(str(dec))
                self.obsInfoDict['dec']=dec
        f.close()




    def getProductsTitle(self):
        controlFrame = QFrame()
        controlFrame.setObjectName("control")
        controlFrame.setMaximumSize(1000, 40)            
        btnWidth = 110
        btnHeight = 24
        btnStartXPos = 220
        btnStartYPos = 5
        #Product titles
        self.eventListProductTitle = QLineEdit(controlFrame)
        self.eventListProductTitle.setText("EventList")
        self.eventListProductTitle.setAlignment(Qt.AlignCenter)
        self.eventListProductTitle.setGeometry(QRect(btnStartXPos,btnStartYPos,btnWidth,btnHeight))
        self.eventListProductTitle.setReadOnly(True)
        self.GTIProductTitle = QLineEdit(controlFrame)
        self.GTIProductTitle.setText("GTI Filtering")
        self.GTIProductTitle.setGeometry(QRect(btnStartXPos+btnWidth,btnStartYPos,btnWidth,btnHeight))
        self.GTIProductTitle.setAlignment(Qt.AlignCenter)
        self.GTIProductTitle.setReadOnly(True)
        #self.pileupProductTitle = QLineEdit(controlFrame)
        #self.pileupProductTitle.setText("Pileup")
        #self.pileupProductTitle.setAlignment(Qt.AlignCenter)
        #self.pileupProductTitle.setGeometry(QRect(btnStartXPos+(2*btnWidth),btnStartYPos,btnWidth,btnHeight))
        #self.pileupProductTitle.setReadOnly(True)
        self.srcDetectProductTitle = QLineEdit(controlFrame)
        self.srcDetectProductTitle.setText("Src Detect")
        self.srcDetectProductTitle.setAlignment(Qt.AlignCenter)        
        self.srcDetectProductTitle.setGeometry(QRect(btnStartXPos+(2*btnWidth),btnStartYPos,btnWidth,btnHeight))
        self.srcDetectProductTitle.setReadOnly(True)
        self.spectraProductTitle = QLineEdit(controlFrame)
        self.spectraProductTitle.setText("Spectra")
        self.spectraProductTitle.setAlignment(Qt.AlignCenter)
        self.spectraProductTitle.setGeometry(QRect(btnStartXPos+(3*btnWidth),btnStartYPos,btnWidth,btnHeight))
        self.spectraProductTitle.setReadOnly(True)
        self.lightCurveProductTitle = QLineEdit(controlFrame)
        self.lightCurveProductTitle.setText("LightCurve")
        self.lightCurveProductTitle.setAlignment(Qt.AlignCenter)
        self.lightCurveProductTitle.setGeometry(QRect(btnStartXPos+(4*btnWidth),btnStartYPos,btnWidth,btnHeight))
        self.lightCurveProductTitle.setReadOnly(True)       
        self.fluxedProductTitle = QLineEdit(controlFrame)
        self.fluxedProductTitle.setText("Fluxed")
        self.fluxedProductTitle.setAlignment(Qt.AlignCenter)
        self.fluxedProductTitle.setGeometry(QRect(btnStartXPos+(5*btnWidth),btnStartYPos,btnWidth,btnHeight))
        self.fluxedProductTitle.setReadOnly(True)       
        
        return controlFrame




    def createControlPanel(self):
        self.cp = None
        self.cp = controlPanel(self.procEPNDict,self.procEMOS1Dict,\
                          self.procEMOS2Dict,self.procRGS1Dict,\
                          self.procRGS2Dict,self.procOMDict)       
        
        
        return self.cp.getFrame()






    def getControlButtons(self,bf):      
        buttonsFrame = QtWidgets.QFrame(bf)
        buttonsFrame.setObjectName("frame")
        buttonsBox = QHBoxLayout(buttonsFrame);        
        buttonsBox.addWidget(self.createRunButton(buttonsFrame));
        buttonsBox.addWidget(self.createCancelButton(buttonsFrame));

        return buttonsFrame

    def createRunButton(self,bf):
        self.runButton = QtWidgets.QPushButton(bf);
        self.runButton.setObjectName("Run");
        if self.getFileName('.cif',str(self.outputField.text())) == "NOT FOUND":            
            self.runButton.setText("Run cifbuild")
        elif self.getFileName('SUM.SAS',str(self.outputField.text())) == "NOT FOUND":
            if self.getFileName('.cif',str(self.outputField.text())) != "NOT FOUND":       
                os.environ['SAS_CCF']=self.getFileName('.cif',str(self.outputField.text()))
                if environ.get('SAS_ODF') is  None:
                    os.environ['SAS_ODF']= str(self.outputField.text())+"/ODF"
            self.runButton.setText("Run odfingest")
        elif self.getFileName('SUM.SAS',str(self.outputField.text())) != "NOT FOUND" \
                and self.getFileName('Param.xml',str(self.outputField.text())) == "NOT FOUND":
            os.environ['SAS_ODF']=self.getFileName('SUM.SAS',str(self.outputField.text())) 
            self.runButton.setText("Run odfParamCreator")        
        #elif self.getFileName('Param.xml',str(self.outputField.text())) == "NOT FOUND":
        #    self.runButton.setText("Run odfParamCreator")        
        else:                                   
            self.runButton.setText("Run xmmextractor")
            self.xmldocFile = self.getFileName("Param.xml", str(self.outputField.text()))
            self.xmldoc = minidom.parse(self.xmldocFile)
            if self.DownloadODFFlag == True:
                self.resetApp("init")
                        
        self.runButton.clicked.connect(self.on_click_ok)
        return self.runButton






    def addInstrumentExpoInfo(self):

        self.odfBrowserFrame = QFrame()
        LabelODFB = QLabel(self.odfBrowserFrame)
        LabelODFB.setText("ODF Browser")
        odfBrowserLayout = QVBoxLayout()
        self.odfBrowserFrame.setLayout(odfBrowserLayout)
        odfBrowserLayout.addWidget(LabelODFB,0)

        scroll = QScrollArea()
        scroll.setWidgetResizable(True) 
        scroll.setVisible(True)
        #scroll.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
       
        InstrumentFrame = QFrame()        
        #InstrumentForm = QFormLayout()
        InstrumentLayout = QVBoxLayout()
        #InstrumentForm.setFormAlignment(Qt.AlignCenter)
        InstrumentFrame.setLayout(InstrumentLayout)

        self.PNButtonGroup = QButtonGroup()
        self.PNButtonGroup.buttonClicked[QAbstractButton].connect(self.on_PNinfoButton_click)         
        PNInstrument = QPushButton("EPN")
        self.PNButtonGroup.addButton(PNInstrument) 
        buttonLayout = QHBoxLayout()
        buttonLayout.setAlignment(Qt.AlignLeft)
        buttonLayout.addWidget(PNInstrument)
        for expid,info in self.EPNExpoInfoDict.items():  
            PNinfoButton = QPushButton(expid)
            PNinfoButton.setFixedWidth(round(float(info['duration'])/50.))
            toolTipString = "Duration: " +info['duration'] +"\n"+"Mode: "+info['mode']
            PNinfoButton.setToolTip(toolTipString)
            self.PNButtonGroup.addButton(PNinfoButton)
            buttonLayout.addWidget(PNinfoButton)
        

        InstrumentLayout.addLayout(buttonLayout)
            
        self.EMOS1ButtonGroup = QButtonGroup()
        self.EMOS1ButtonGroup.buttonClicked[QAbstractButton].connect(self.on_EMOS1infoButton_click)   
        EMOS1Instrument = QPushButton("EMOS1")
        self.EMOS1ButtonGroup.addButton(EMOS1Instrument) 
        buttonLayout = QHBoxLayout()
        buttonLayout.setAlignment(Qt.AlignLeft)
        buttonLayout.addWidget(EMOS1Instrument)
        for expid,info in self.EMOS1ExpoInfoDict.items():                        
            EMOS1infoButton = QPushButton(expid)
            EMOS1infoButton.setFixedWidth(round(float(info['duration'])/50.))
            toolTipString = "Duration: " +info['duration'] +"\n"+"Mode: "+info['mode']
            EMOS1infoButton.setToolTip(toolTipString)
            self.EMOS1ButtonGroup.addButton(EMOS1infoButton)
            buttonLayout.addWidget(EMOS1infoButton)

        InstrumentLayout.addLayout(buttonLayout)
  
        self.EMOS2ButtonGroup = QButtonGroup()
        self.EMOS2ButtonGroup.buttonClicked[QAbstractButton].connect(self.on_EMOS2infoButton_click)   
        EMOS2Instrument = QPushButton("EMOS2")
        self.EMOS2ButtonGroup.addButton(EMOS2Instrument) 
        buttonLayout = QHBoxLayout()
        buttonLayout.setAlignment(Qt.AlignLeft)
        buttonLayout.addWidget(EMOS2Instrument)
        for expid,info in self.EMOS2ExpoInfoDict.items():                        
            EMOS2infoButton = QPushButton(expid)
            EMOS2infoButton.setFixedWidth(round(float(info['duration'])/50.))
            toolTipString = "Duration: " +info['duration'] +"\n"+"Mode: "+info['mode']
            EMOS2infoButton.setToolTip(toolTipString)
            self.EMOS2ButtonGroup.addButton(EMOS2infoButton)
            buttonLayout.addWidget(EMOS2infoButton)            

        InstrumentLayout.addLayout(buttonLayout)

        self.RGS1ButtonGroup = QButtonGroup()
        self.RGS1ButtonGroup.buttonClicked[QAbstractButton].connect(self.on_RGS1infoButton_click)        
        RGS1Instrument = QPushButton("RGS1")
        self.RGS1ButtonGroup.addButton(RGS1Instrument) 
        buttonLayout = QHBoxLayout()
        buttonLayout.setAlignment(Qt.AlignLeft)
        buttonLayout.addWidget(RGS1Instrument)
        for expid,info in self.RGS1ExpoInfoDict.items():                        
            RGS1infoButton = QPushButton(expid)
            RGS1infoButton.setFixedWidth(round(float(info['duration'])/50.))
            toolTipString = "Duration: " +info['duration'] +"\n"+"Mode: "+info['mode']
            RGS1infoButton.setToolTip(toolTipString)
            self.RGS1ButtonGroup.addButton(RGS1infoButton)
            buttonLayout.addWidget(RGS1infoButton)

        InstrumentLayout.addLayout(buttonLayout)

        self.RGS2ButtonGroup = QButtonGroup()
        self.RGS2ButtonGroup.buttonClicked[QAbstractButton].connect(self.on_RGS2infoButton_click)
        RGS2Instrument = QPushButton("RGS2")
        self.RGS2ButtonGroup.addButton(RGS2Instrument)
        buttonLayout = QHBoxLayout()
        buttonLayout.setAlignment(Qt.AlignLeft)
        buttonLayout.addWidget(RGS2Instrument)
        for expid,info in self.RGS2ExpoInfoDict.items():                        
            RGS2infoButton = QPushButton(expid)
            RGS2infoButton.setFixedWidth(round(float(info['duration'])/50.))
            toolTipString = "Duration: " +info['duration'] +"\n"+"Mode: "+info['mode']
            RGS2infoButton.setToolTip(toolTipString)
            self.RGS2ButtonGroup.addButton(RGS2infoButton)
            buttonLayout.addWidget(RGS2infoButton)
        
        InstrumentLayout.addLayout(buttonLayout)

        self.OMButtonGroup = QButtonGroup()
        self.OMButtonGroup.buttonClicked[QAbstractButton].connect(self.on_OMinfoButton_click)
        OMInstrument = QPushButton("OM")
        self.OMButtonGroup.addButton(OMInstrument)
        buttonLayout = QHBoxLayout()
        buttonLayout.setAlignment(Qt.AlignLeft)
        buttonLayout.addWidget(OMInstrument)
        for expid,info in self.OMExpoInfoDict.items():                        
            OMinfoButton = QPushButton(expid)
            OMinfoButton.setFixedWidth(round(float(info['duration'])/30.))
            toolTipString = "Duration: " +info['duration'] +"\n"+"Mode: "+info['mode']
            OMinfoButton.setToolTip(toolTipString)
            self.OMButtonGroup.addButton(OMinfoButton)
            buttonLayout.addWidget(OMinfoButton)
            

        InstrumentLayout.addLayout(buttonLayout)
        scroll.setWidget(InstrumentFrame)
        
        #return scroll
        odfBrowserLayout.addWidget(scroll,4)
        return self.odfBrowserFrame
      




    def on_PNinfoButton_click(self,bt):
        self.logger.info("Clicked " + bt.text())
    def on_EMOS1infoButton_click(self,bt):
        self.logger.info("Clicked " + bt.text())
    def on_EMOS2infoButton_click(self,bt):
        self.logger.info("Clicked " + bt.text())
    def on_RGS1infoButton_click(self,bt):
        self.logger.info("Clicked " + bt.text())
    def on_RGS2infoButton_click(self,bt):
        self.logger.info("Clicked " + bt.text())
    def on_OMinfoButton_click(self,bt):
        self.logger.info("Clicked " + bt.text())

    # Create the actions
    @pyqtSlot()    
    def on_click(self):
        #self.logger.info("Exiting...")
        #delete XML template file
        
        if  Path(self.templateXMLFileName).is_file():
           os.remove(self.templateXMLFileName)

        if self.pid is not None:
            try:
                #self.logger.info("Killing process "+str(self.pid))     
                
                os.kill(self.pid, signal.SIGTERM)
            except OSError:
                print("No process to stop...")     

        if self.p is not None:
            if self.p.pid.poll() is None :
                self.p.set('exit')

        QApplication.instance().exit()


        
    @pyqtSlot()
    def on_click_ok(self):     
        
        #prepare the processing paramaters for each specific analysis.
        #If user is in the "Setting" tab, it will execute xmmextractor as a whole
        #If user is in a particular tab, it will execute xmmextractor tailored for 
        #an specfic product.
        instrument = "" 
        expoID = ""
        currentProductLabel = "" 

        if (self.tabs.currentIndex() > 1 ):
            instrument = self.tabs.tabText(self.tabs.currentIndex())
            expoID =  self.tabs.currentWidget().tabText( self.tabs.currentWidget().currentIndex())
            currentExpoTab = self.expoTabProductDict[str(expoID)]
            currentProductLabel=self.expoTabProductDict[str(expoID)].tabText(self.expoTabProductDict[str(expoID)].currentIndex()) 

            #... AI Check this calls. Maybe not necessaries any longer 
            self.setCurrentProcessingParam(self.procEPNDict,expoID,currentProductLabel,instrument)
            self.setCurrentProcessingParam( self.procEMOS1Dict,expoID,currentProductLabel,instrument)
            self.setCurrentProcessingParam( self.procEMOS2Dict,expoID,currentProductLabel,instrument)
            self.setCurrentProcessingParam( self.procRGS1Dict,expoID,currentProductLabel,instrument)
            self.setCurrentProcessingParam( self.procRGS2Dict,expoID,currentProductLabel,instrument)
            self.setCurrentProcessingParam( self.procOMDict,expoID,currentProductLabel,instrument)


        self.XMMTimer = QTimer()

        #self.logger.info("Create XML file")
        #... Control log...
        #Parse the epicsrc field and write 'yes' or 'no;
        if  self.odf_epicsr_data.currentText() == 'Proposal Central Target' or self.odf_epicsr_data.currentText() == 'User\'s Defined':
            if len(self.odf_ra_data.text()) == 0 or len(self.odf_dec_data.text()) == 0:
                msg = QMessageBox()
                msg.setWindowTitle("Coordinates")
                msg.setIcon(QMessageBox.Warning)
                msg.setText("Empty RA or Dec fields")
                msg.setStandardButtons(QMessageBox.Ok)
                msg.buttonClicked.connect(self.msgbtn)
                msg.exec_()
                return

            self.obsInfoDict['epicsrc'] = 'yes'                
        else:
            self.obsInfoDict['epicsrc'] = 'no'
        self.obsInfoDict['ra'] = self.odf_ra_data.text()
        self.obsInfoDict['dec'] = self.odf_dec_data.text()
        xmlfile = createXML(self.procEPNDict,self.procEMOS1Dict,\
                            self.procEMOS2Dict,self.procRGS1Dict,\
                            self.procRGS2Dict,self.procOMDict,self.obsInfoDict,\
                            self.tasksEPNDict,self.tasksEMOS1Dict,\
                            self.tasksEMOS2Dict,self.tasksRGS1Dict,\
                            self.tasksRGS2Dict,self.tasksOMDict,\
                            self.xmldocFile)

        os.chdir(self.outputField.text())
        if str(self.runButton.text()).find('cifbuild') != -1:
            #self.logger.info("Ready to run cifbuild")
            executeJob = doIt('')
            executeJob.runSASCommand('cifbuild')
        elif str(self.runButton.text()).find('odfingest') != -1:
            #self.logger.info("Ready to run odfingest")
            executeJob = doIt('')            
            executeJob.runSASCommand('odfingest')
        elif str(self.runButton.text()).find('odfParamCreator') != -1:
            #self.logger.info("Ready to run odfParamCreator")
            executeJob = doIt('')
            executeJob.runSASCommand('odfParamCreator')
        else:
            #self.logger.info("Ready to run xmmextractor")
            executeJob = doIt("xmmextractorParam.xml")
            executeJob.runSASCommand('xmmextractor')

        #executeJob.checkJobStatus()    
        #self.logger.info("Starting New Job...")
        self.runButton.setEnabled(False)
        
        self.prod=""
        #get PID 
        self.pid = executeJob.getPID()
        #self.logger.info("Current PID number " + str(self.pid))
        executeJob.setPID(self.pid)

        timerCallback = functools.partial(self.f, executeJob,self.runButton,self.pid)
        self.XMMTimer.timeout.connect(timerCallback)
        self.XMMTimer.start(1000)

        #self.f( executeJob,self.runButton, pid)

    def createCancelButton(self,bf):
        self.cancelButton = QtWidgets.QPushButton(bf);
        self.cancelButton.setObjectName("Exit");
        #cancelButton.setGeometry(QRect(600, 30, 80, 22))
        self.cancelButton.setText("Exit");
        #cancelButton.clicked.connect(QCoreApplication.instance().quit);
        self.cancelButton.clicked.connect(self.on_click);
        return self.cancelButton              

    def msgbtn(i):
        print ("Button pressed is:"+str(i))

    def handleLogin(self):
        print("USER "+ str(self.userName.text()))
        if self.userName.text() != "":
            os.environ["AIOUSER"]=str(self.userName.text())
            os.environ["AIOPWD"]=str(self.passWord.text())
        
        self.loginWidget.accept()

    def handleCancel(self):
        self.loginWidget.reject()



    def tabSelected(self,arg=None):
        #self.logger.info("Current Tab Title " + str(self.tabs.tabText( arg)))

        if arg > 1:
            #self.logger.info("Current Tab Index: "+str(arg))       
            instName = self.tabs.tabText( self.tabs.currentIndex())
            expoTab = self.tabs.currentWidget()           

            expoID = expoTab.tabText(expoTab.currentIndex())
            currentExpoTab = self.expoTabProductDict[str(expoID)]
            currentProductLabel=currentExpoTab.tabText(currentExpoTab.currentIndex()) 

            text = "Create "+currentProductLabel+" for " + instName+ " exposure "+str(expoID)
            self.runButton.setText(text)           
            if (self.checkProductGeneration(currentProductLabel,instName,str(expoID)) == False):
                self.runButton.setEnabled(False)
        else:
            #self.logger.info("Settings tab.... "+ str(arg))          
            if self.getFileName('.cif',str(self.outputField.text())) == "NOT FOUND":            
                    self.runButton.setText("Run cifbuild")
            elif self.getFileName('SUM.SAS',str(self.outputField.text())) == "NOT FOUND":
                if self.getFileName('.cif',str(self.outputField.text())) != "NOT FOUND":       
                    os.environ['SAS_CCF']=self.getFileName('.cif',str(self.outputField.text()))                      
                self.runButton.setText("Run odfingest")
            elif self.getFileName('Param.xml',str(self.outputField.text())) == "NOT FOUND":   
                if self.getFileName('SUM.SAS',str(self.outputField.text())) != "NOT FOUND":
                    os.environ['SAS_ODF']=self.getFileName('SUM.SAS',str(self.outputField.text())) 
                self.runButton.setText("Run odfParamCreator")
            else:
                self.runButton.setText("Run xmmextractor")
            self.runButton.setEnabled(True)
            

    #TO DO: TO BE DELETED AND REPLACE BY xmmextrator_tools (AG)
    def unpackODF(self):
        #self.logger.info("Unpacking ODF...")
        #Create ODF directory
        dirPath = str(self.outputField.text())
        #self.logger.info("Creating ObsId directory... " + dirPath)
        path = Path(dirPath)
        path.mkdir(exist_ok=True)
        odfDir = dirPath+'/ODF' 
        #self.logger.info("Creating ODF directory... " + odfDir)
        path = Path(dirPath)
        path.mkdir(exist_ok=True) 
        

        #self.logger.info("untar ODF file...")
        tgzName = str(self.outputField.text())+'/'+str(self.odf_obsid_data.text())+'.tar.gz'

        try:            
            tar_file = tarfile.open(tgzName,'r:gz')
        except:
            message = "Could not open tar file: \n"\
                " The file probably does not have the correct format.\n"\
                " --> Inner message:"
            raise Exception(message)
                          
        tar_file.extractall(odfDir)
        #os.chdir(odfDir)        
        tarName= str(self.getFileName(".TAR",odfDir))
        #self.logger.info("TAR File "+ tarName)
        tar_file = tarfile.open(tarName,'r')
        tar_file.extractall(odfDir)
        tar_file.close()
        os.environ["SAS_ODF"]=odfDir
        #self.logger.info("ODF unpacked")
        self.runButton.setDisabled(False)                
        if os.path.isfile(tarName):
            os.remove(tarName)
            os.remove(tgzName)
        else:
            print("Cannot Remove"+tarName+" file")

            
# iparsdic is a dictionary with all the task parameters, where their
# respective values are either those entered from the command line 
# or those taken from par file defaults.

def run(iparsdic):
    print(f'Executing {__file__} {iparsdic}')

    xmldocFile = [iparsdic['xml']]
    #xmldocFile = ['none']
    
    #xmldocFile = ['0810862501.xml']
        
    app = QApplication(xmldocFile)

    #XMMTimer = QTimer()

    ex = App(xmldocFile)
    sys.exit(app.exec_())
        

    
